<?php
if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

/**
 * Admin Pages class
 * Handles all admin pages for viewing upcoming and recurring restocks
 */
class NCWCAPR_Admin_Pages {

    /**
     * Constructor
     */
    public function __construct() {
        // Add admin pages to the menu
        add_action('admin_menu', array($this, 'add_admin_pages'));
        
        // Add admin notices
        add_action('admin_notices', array($this, 'admin_notices'));
    }
    
    /**
     * Add admin pages to the menu
     */
    public function add_admin_pages() {
        // Upcoming restocks page
        add_submenu_page(
            'edit.php?post_type=product',
            esc_html__('Upcoming Restocks', 'auto-product-restock-for-woo'),
            esc_html__('Upcoming Restocks', 'auto-product-restock-for-woo'),
            'manage_woocommerce',
            'apr-upcoming-restocks',
            array($this, 'upcoming_restocks_page')
        );
        
        // Recurring restocks page
        add_submenu_page(
            'edit.php?post_type=product',
            esc_html__('Recurring Restocks', 'auto-product-restock-for-woo'),
            esc_html__('Recurring Restocks', 'auto-product-restock-for-woo'),
            'manage_woocommerce',
            'apr-recurring-restocks',
            array($this, 'recurring_restocks_page')
        );
    }
    
    /**
     * Display upcoming restocks page
     */
    public function upcoming_restocks_page() {
        // Get current date
        $today = gmdate('Y-m-d');
        $thirty_days_later = gmdate('Y-m-d', strtotime('+30 days'));
        
        // Query one-time restocks in the next 30 days
        $args = array(
            'post_type' => 'product',
            'post_status' => 'publish',
            'posts_per_page' => -1,
            'meta_query' => array(
                array(
                    'key' => '_restock_date',
                    'value' => array($today, $thirty_days_later),
                    'compare' => 'BETWEEN',
                    'type' => 'DATE'
                ),
            ),
            'orderby' => 'meta_value',
            'meta_key' => '_restock_date',
            'order' => 'ASC'
        );
        
        $upcoming_restocks = new WP_Query($args);
        
        ?>
        <div class="wrap">
            <h1 class="wp-heading-inline"><?php esc_html_e('Upcoming Product Restocks', 'auto-product-restock-for-woo'); ?></h1>
            <hr class="wp-header-end">
            
            <div class="notice notice-info inline">
                <p><?php esc_html_e('This page shows products scheduled to be restocked in the next 30 days.', 'auto-product-restock-for-woo'); ?></p>
            </div>
            
            <?php if ($upcoming_restocks->have_posts()) : ?>
                <table class="wp-list-table widefat fixed striped posts">
                    <thead>
                        <tr>
                            <th scope="col" class="manage-column column-title column-primary"><?php esc_html_e('Product', 'auto-product-restock-for-woo'); ?></th>
                            <th scope="col" class="manage-column"><?php esc_html_e('Current Stock', 'auto-product-restock-for-woo'); ?></th>
                            <th scope="col" class="manage-column"><?php esc_html_e('Restock Date & Time', 'auto-product-restock-for-woo'); ?></th>
                            <th scope="col" class="manage-column"><?php esc_html_e('Restock Quantity', 'auto-product-restock-for-woo'); ?></th>
                        </tr>
                    </thead>
                    <tbody>
                        <?php while ($upcoming_restocks->have_posts()) : $upcoming_restocks->the_post(); 
                            $product_id = get_the_ID();
                            $product = wc_get_product($product_id);
                            
                            if (!$product) {
                                continue;
                            }
                            
                            $restock_date = get_post_meta($product_id, '_restock_date', true);
                            $restock_quantity = get_post_meta($product_id, '_restock_quantity', true);
                            $current_stock = $product->get_stock_quantity();
                            $edit_link = get_edit_post_link($product_id);
                            $view_link = get_permalink($product_id);
                            
                            // Format the datetime
                            $formatted_date = date_i18n(get_option('date_format') . ' ' . get_option('time_format'), strtotime($restock_date));
                        ?>
                            <tr>
                                <td class="title column-title has-row-actions column-primary">
                                    <strong><a href="<?php echo esc_url($edit_link); ?>"><?php echo esc_html($product->get_name()); ?></a></strong>
                                    <div class="row-actions">
                                        <span class="edit"><a href="<?php echo esc_url($edit_link); ?>"><?php esc_html_e('Edit', 'auto-product-restock-for-woo'); ?></a> | </span>
                                        <span class="view"><a href="<?php echo esc_url($view_link); ?>"><?php esc_html_e('View', 'auto-product-restock-for-woo'); ?></a></span>
                                    </div>
                                </td>
                                <td><?php echo esc_html($current_stock !== null ? $current_stock : esc_html__('N/A', 'auto-product-restock-for-woo')); ?></td>
                                <td><?php echo esc_html($formatted_date); ?></td>
                                <td><?php echo esc_html($restock_quantity); ?></td>
                            </tr>
                        <?php endwhile; ?>
                    </tbody>
                </table>
            <?php else : ?>
                <div class="notice notice-warning inline">
                    <p><?php esc_html_e('No upcoming product restocks in the next 30 days.', 'auto-product-restock-for-woo'); ?></p>
                </div>
            <?php endif; ?>
            
            <?php wp_reset_postdata(); ?>
        </div>
        <?php
    }
    
    /**
     * Display recurring restocks page
     */
    public function recurring_restocks_page() {
        // Query products with recurring restocks enabled
        $args = array(
            'post_type' => 'product',
            'post_status' => 'publish',
            'posts_per_page' => -1,
            'meta_query' => array(
                array(
                    'key' => '_restock_recurrence_enabled',
                    'value' => 'yes',
                    'compare' => '='
                ),
            ),
            'orderby' => 'title',
            'order' => 'ASC'
        );
        
        $recurring_restocks = new WP_Query($args);
        
        ?>
        <div class="wrap">
            <h1 class="wp-heading-inline"><?php esc_html_e('Recurring Product Restocks', 'auto-product-restock-for-woo'); ?></h1>
            <hr class="wp-header-end">
            
            <div class="notice notice-info inline">
                <p><?php esc_html_e('This page shows products configured for recurring restocks.', 'auto-product-restock-for-woo'); ?></p>
            </div>
            
            <?php if ($recurring_restocks->have_posts()) : ?>
                <table class="wp-list-table widefat fixed striped posts">
                    <thead>
                        <tr>
                            <th scope="col" class="manage-column column-title column-primary"><?php esc_html_e('Product', 'auto-product-restock-for-woo'); ?></th>
                            <th scope="col" class="manage-column"><?php esc_html_e('Current Stock', 'auto-product-restock-for-woo'); ?></th>
                            <th scope="col" class="manage-column"><?php esc_html_e('Recurrence Pattern', 'auto-product-restock-for-woo'); ?></th>
                            <th scope="col" class="manage-column"><?php esc_html_e('Restock Time', 'auto-product-restock-for-woo'); ?></th>
                            <th scope="col" class="manage-column"><?php esc_html_e('Restock Quantity', 'auto-product-restock-for-woo'); ?></th>
                            <th scope="col" class="manage-column"><?php esc_html_e('Last Restocked', 'auto-product-restock-for-woo'); ?></th>
                        </tr>
                    </thead>
                    <tbody>
                        <?php while ($recurring_restocks->have_posts()) : $recurring_restocks->the_post(); 
                            $product_id = get_the_ID();
                            $product = wc_get_product($product_id);
                            
                            if (!$product) {
                                continue;
                            }
                            
                            $recurrence_type = get_post_meta($product_id, '_restock_recurrence_type', true);
                            $recurrence_time = get_post_meta($product_id, '_restock_recurrence_time', true);
                            $restock_quantity = get_post_meta($product_id, '_restock_quantity', true);
                            $current_stock = $product->get_stock_quantity();
                            $last_restock = get_post_meta($product_id, '_last_recurring_restock', true);
                            $edit_link = get_edit_post_link($product_id);
                            $view_link = get_permalink($product_id);
                            
                            // Format the recurrence info
                            $recurrence_info = esc_html__('Unknown', 'auto-product-restock-for-woo');
                            
                            if ($recurrence_type === 'weekly') {
                                $days = array();
                                $day_names = array(
                                    'monday' => esc_html__('Monday', 'auto-product-restock-for-woo'),
                                    'tuesday' => esc_html__('Tuesday', 'auto-product-restock-for-woo'),
                                    'wednesday' => esc_html__('Wednesday', 'auto-product-restock-for-woo'),
                                    'thursday' => esc_html__('Thursday', 'auto-product-restock-for-woo'),
                                    'friday' => esc_html__('Friday', 'auto-product-restock-for-woo'),
                                    'saturday' => esc_html__('Saturday', 'auto-product-restock-for-woo'),
                                    'sunday' => esc_html__('Sunday', 'auto-product-restock-for-woo')
                                );
                                
                                foreach ($day_names as $day_key => $day_value) {
                                    if (get_post_meta($product_id, '_restock_day_' . $day_key, true) === 'yes') {
                                        $days[] = $day_value;
                                    }
                                }
                                
                                $recurrence_info = esc_html__('Weekly', 'auto-product-restock-for-woo') . ': ' . implode(', ', $days);
                            } elseif ($recurrence_type === 'monthly') {
                                $day_of_month = get_post_meta($product_id, '_restock_day_of_month', true);
                                $recurrence_info = sprintf(
                                    /* translators: %d: Day of month (1-31) */
                                    esc_html__('Monthly: Day %d', 'auto-product-restock-for-woo'),
                                    intval($day_of_month)
                                );
                            }
                            
                            // Format the last restock date
                            $formatted_last_restock = $last_restock ? date_i18n(get_option('date_format'), strtotime($last_restock)) : esc_html__('Never', 'auto-product-restock-for-woo');
                            
                            // Format the time
                            $formatted_time = !empty($recurrence_time) ? date_i18n(get_option('time_format'), strtotime($recurrence_time)) : esc_html__('Midnight', 'auto-product-restock-for-woo');
                        ?>
                            <tr>
                                <td class="title column-title has-row-actions column-primary">
                                    <strong><a href="<?php echo esc_url($edit_link); ?>"><?php echo esc_html($product->get_name()); ?></a></strong>
                                    <div class="row-actions">
                                        <span class="edit"><a href="<?php echo esc_url($edit_link); ?>"><?php esc_html_e('Edit', 'auto-product-restock-for-woo'); ?></a> | </span>
                                        <span class="view"><a href="<?php echo esc_url($view_link); ?>"><?php esc_html_e('View', 'auto-product-restock-for-woo'); ?></a></span>
                                    </div>
                                </td>
                                <td><?php echo esc_html($current_stock !== null ? $current_stock : esc_html__('N/A', 'auto-product-restock-for-woo')); ?></td>
                                <td><?php echo esc_html($recurrence_info); ?></td>
                                <td><?php echo esc_html($formatted_time); ?></td>
                                <td><?php echo esc_html($restock_quantity); ?></td>
                                <td><?php echo esc_html($formatted_last_restock); ?></td>
                            </tr>
                        <?php endwhile; ?>
                    </tbody>
                </table>
            <?php else : ?>
                <div class="notice notice-warning inline">
                    <p><?php esc_html_e('No products with recurring restocks configured.', 'auto-product-restock-for-woo'); ?></p>
                </div>
            <?php endif; ?>
            
            <?php wp_reset_postdata(); ?>
        </div>
        <?php
    }
    
        /**
        * Display admin notices
        * 
        * @return void
        */
        public function admin_notices() {
            // This is just checking URL parameters for displaying notices, not performing actions
            // phpcs:ignore WordPress.Security.NonceVerification.Recommended
            if (isset($_GET['page']) && $_GET['page'] === 'apr-auto-restock-settings' && isset($_GET['synced']) && $_GET['synced'] == '1') {
                ?>
                <div class="notice notice-success is-dismissible">
                    <p><?php esc_html_e('Stock check completed successfully.', 'auto-product-restock-for-woo'); ?></p>
                </div>
                <?php
            }
        }
}