<?php
if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

/**
 * Settings class for Auto Product Restock
 * Handles plugin settings and synchronization
 */
class NCWCAPR_Auto_Restock_Settings {

    /**
     * Constructor
     */
    public function __construct() {
        // Settings page in admin menu
        add_action('admin_menu', array($this, 'add_settings_page'));
        add_action('admin_init', array($this, 'register_settings'));
        
        // Hooks for settings updates
        add_action('update_option_ncwcapr_restock_cron_schedule', array($this, 'update_cron_schedule'), 10, 2);
        add_action('admin_post_apr_restock_sync', array($this, 'handle_sync_now'));
    }

    /**
     * Add settings page to menu
     */
    public function add_settings_page() {
        add_submenu_page(
            'woocommerce',
            esc_html__('Auto Restock Settings', 'auto-product-restock-for-woo'),
            esc_html__('Auto Restock Settings', 'auto-product-restock-for-woo'),
            'manage_options',
            'apr-auto-restock-settings',
            array($this, 'settings_page')
        );
    }

    /**
     * Register plugin settings
     */
    public function register_settings() {
        // Register settings
        register_setting(
            'ncwcapr_restock_settings', 
            'ncwcapr_restock_display_position', 
            array(
                'sanitize_callback' => 'sanitize_text_field',
                'default' => 'woocommerce_single_product_summary',
                'type' => 'string'
            )
        );

        register_setting(
            'ncwcapr_restock_settings', 
            'ncwcapr_restock_cron_schedule', 
            array(
                'sanitize_callback' => array($this, 'sanitize_positive_int'),
                'default' => 4,
                'type' => 'integer'
            )
        );

        register_setting(
            'ncwcapr_restock_settings', 
            'ncwcapr_restock_enable_logging', 
            array(
                'sanitize_callback' => 'sanitize_text_field',
                'default' => 'no',
                'type' => 'string'
            )
        );

        // Add settings section
        add_settings_section(
            'ncwcapr_restock_settings_section',
            esc_html__('Auto Restock Settings', 'auto-product-restock-for-woo'),
            null,
            'apr-auto-restock-settings'
        );

        // Add settings fields
        add_settings_field(
            'ncwcapr_restock_display_position',
            esc_html__('Display Position', 'auto-product-restock-for-woo'),
            array($this, 'display_position_field'),
            'apr-auto-restock-settings',
            'ncwcapr_restock_settings_section'
        );

        add_settings_field(
            'ncwcapr_restock_cron_schedule',
            esc_html__('Check Frequency', 'auto-product-restock-for-woo'),
            array($this, 'cron_schedule_field'),
            'apr-auto-restock-settings',
            'ncwcapr_restock_settings_section'
        );
        
        add_settings_field(
            'ncwcapr_restock_enable_logging',
            esc_html__('Enable Logging', 'auto-product-restock-for-woo'),
            array($this, 'enable_logging_field'),
            'apr-auto-restock-settings',
            'ncwcapr_restock_settings_section'
        );
    }
    
    /**
     * Sanitize positive integer values
     */
    public function sanitize_positive_int($value) {
        $value = absint($value);
        return $value > 0 ? $value : 4; // Default to 4 if invalid
    }

    /**
     * Display position field
     */
    public function display_position_field() {
        $options = get_option('ncwcapr_restock_display_position', 'woocommerce_single_product_summary');
        ?>
        <select name="ncwcapr_restock_display_position">
            <option value="woocommerce_single_product_summary" <?php selected($options, 'woocommerce_single_product_summary'); ?>><?php esc_html_e('After Product Summary', 'auto-product-restock-for-woo'); ?></option>
            <option value="woocommerce_before_single_product_summary" <?php selected($options, 'woocommerce_before_single_product_summary'); ?>><?php esc_html_e('Before Product Summary', 'auto-product-restock-for-woo'); ?></option>
            <option value="woocommerce_product_meta_start" <?php selected($options, 'woocommerce_product_meta_start'); ?>><?php esc_html_e('Before Product Meta', 'auto-product-restock-for-woo'); ?></option>
            <option value="woocommerce_product_meta_end" <?php selected($options, 'woocommerce_product_meta_end'); ?>><?php esc_html_e('After Product Meta', 'auto-product-restock-for-woo'); ?></option>
        </select>
        <p class="description"><?php esc_html_e('Choose where to display the expected restock date on product pages.', 'auto-product-restock-for-woo'); ?></p>
        <?php
    }

    /**
     * Check frequency field
     */
    public function cron_schedule_field() {
        $options = get_option('ncwcapr_restock_cron_schedule', 4); // Default to 4 hours
        ?>
        <input type="number" name="ncwcapr_restock_cron_schedule" value="<?php echo esc_attr($options); ?>" min="1" />
        <p class="description">
            <?php esc_html_e('Enter the number of hours between each stock check. Default is 4 hours.', 'auto-product-restock-for-woo'); ?>
            <br>
            <a href="<?php echo esc_url(wp_nonce_url(admin_url('admin-post.php?action=apr_restock_sync'), 'apr_restock_sync_nonce')); ?>" class="button"><?php esc_html_e('Check Now', 'auto-product-restock-for-woo'); ?></a>
        </p>
        <?php
    }
    
    /**
     * Enable logging field
     */
    public function enable_logging_field() {
        $option = get_option('ncwcapr_restock_enable_logging', 'no');
        ?>
        <label>
            <input type="checkbox" name="ncwcapr_restock_enable_logging" value="yes" <?php checked($option, 'yes'); ?> />
            <?php esc_html_e('Enable debug logging', 'auto-product-restock-for-woo'); ?>
        </label>
        <p class="description"><?php esc_html_e('If enabled, debug information will be written to the error log.', 'auto-product-restock-for-woo'); ?></p>
        <?php
    }

    /**
     * Settings page content
     */
    public function settings_page() {
        ?>
        <div class="wrap">
            <h1><?php esc_html_e('Auto Restock Settings', 'auto-product-restock-for-woo'); ?></h1>
            
            <form method="post" action="options.php">
                <?php
                settings_fields('ncwcapr_restock_settings');
                do_settings_sections('apr-auto-restock-settings');
                submit_button();
                ?>
            </form>
            
            <hr>
            
            <h2><?php esc_html_e('Restock Features', 'auto-product-restock-for-woo'); ?></h2>
            
            <div class="metabox-holder">
                <div class="postbox">
                    <h3 class="hndle"><span><?php esc_html_e('Time-Based Restocking', 'auto-product-restock-for-woo'); ?></span></h3>
                    <div class="inside">
                        <p><?php esc_html_e('Products can now be restocked at a specific time on the specified date, not just at the beginning of the day.', 'auto-product-restock-for-woo'); ?></p>
                        <p><strong><?php esc_html_e('How to use:', 'auto-product-restock-for-woo'); ?></strong></p>
                        <ol>
                            <li><?php esc_html_e('Edit a product and go to the Inventory tab', 'auto-product-restock-for-woo'); ?></li>
                            <li><?php esc_html_e('Set a restock date and time in the format YYYY-MM-DD HH:MM:SS', 'auto-product-restock-for-woo'); ?></li>
                            <li><?php esc_html_e('The product will be restocked automatically when that time is reached', 'auto-product-restock-for-woo'); ?></li>
                        </ol>
                    </div>
                </div>
                
                <div class="postbox">
                    <h3 class="hndle"><span><?php esc_html_e('Recurring Restock Patterns', 'auto-product-restock-for-woo'); ?></span></h3>
                    <div class="inside">
                        <p><?php esc_html_e('Set products to automatically restock on a recurring basis - weekly or monthly.', 'auto-product-restock-for-woo'); ?></p>
                        <p><strong><?php esc_html_e('Weekly pattern:', 'auto-product-restock-for-woo'); ?></strong> <?php esc_html_e('Choose specific days of the week for restocking', 'auto-product-restock-for-woo'); ?></p>
                        <p><strong><?php esc_html_e('Monthly pattern:', 'auto-product-restock-for-woo'); ?></strong> <?php esc_html_e('Choose a specific day of the month for restocking', 'auto-product-restock-for-woo'); ?></p>
                        <p>
                            <a href="<?php echo esc_url(admin_url('edit.php?post_type=product&page=apr-recurring-restocks')); ?>" class="button"><?php esc_html_e('View Recurring Restocks', 'auto-product-restock-for-woo'); ?></a>
                        </p>
                    </div>
                </div>
                
                <div class="postbox">
                    <h3 class="hndle"><span><?php esc_html_e('Dashboard Widget', 'auto-product-restock-for-woo'); ?></span></h3>
                    <div class="inside">
                        <p><?php esc_html_e('A widget has been added to your WordPress dashboard showing:', 'auto-product-restock-for-woo'); ?></p>
                        <ul>
                            <li><?php esc_html_e('Upcoming one-time restocks in the next 30 days', 'auto-product-restock-for-woo'); ?></li>
                            <li><?php esc_html_e('Products configured for recurring restocks', 'auto-product-restock-for-woo'); ?></li>
                        </ul>
                        <p>
                            <a href="<?php echo esc_url(admin_url('edit.php?post_type=product&page=apr-upcoming-restocks')); ?>" class="button"><?php esc_html_e('View Upcoming Restocks', 'auto-product-restock-for-woo'); ?></a>
                        </p>
                    </div>
                </div>
            </div>
        </div>
        <?php
    }

    /**
     * Update cron schedule when setting is changed
     */
    public function update_cron_schedule($old_value, $value) {
        if ($old_value !== $value) {
            wp_clear_scheduled_hook('ncwcapr_restock_check_dates');
            if (!wp_next_scheduled('ncwcapr_restock_check_dates')) {
                wp_schedule_event(time(), 'ncwcapr_restock_interval', 'ncwcapr_restock_check_dates');
            }
        }
    }

    /**
     * Handle manual synchronization
     */
    public function handle_sync_now() {
        if (!current_user_can('manage_options')) {
            return;
        }
        
        // Verify nonce
        if (!isset($_GET['_wpnonce']) || !wp_verify_nonce(sanitize_text_field(wp_unslash($_GET['_wpnonce'])), 'apr_restock_sync_nonce')) {
            wp_die(esc_html__('Security check failed.', 'auto-product-restock-for-woo'));
        }
        
        // Trigger the restock check action
        do_action('ncwcapr_restock_check_dates');
        
        // Redirect back to settings page with success message
        wp_redirect(add_query_arg('synced', '1', admin_url('admin.php?page=apr-auto-restock-settings')));
        exit;
    }
}