<?php
if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

/**
 * Main Auto Restock class
 * Handles core functionality for restocking products
 */
class NCWCAPR_Auto_Restock {

    /**
     * Constructor
     */
    public function __construct() {
        // Hook to add custom product field
        add_action('woocommerce_product_options_inventory_product_data', array($this, 'add_restock_date_field'));
        add_action('woocommerce_process_product_meta', array($this, 'save_restock_date_field'));

        // Enqueue date picker scripts and styles
        add_action('admin_enqueue_scripts', array($this, 'enqueue_date_picker'));

        // Hook to schedule the event
        add_action('wp', array($this, 'schedule_restock_event'));

        // Hook to handle the scheduled event
        add_action('ncwcapr_restock_check_dates', array($this, 'update_stock_status'));

        // Hook to display restock date on the product page
        $display_position = get_option('ncwcapr_restock_display_position', 'woocommerce_single_product_summary');
        add_action($display_position, array($this, 'display_restock_date'), 20);

        // Add custom cron interval
        add_filter('cron_schedules', array($this, 'add_cron_intervals'));
        
        // Store reference to this instance in global
        $GLOBALS['ncwcapr_auto_restock'] = $this;
    }

    /**
     * Enqueue scripts and styles for the date and time picker
     */
    public function enqueue_date_picker() {
        global $pagenow, $post_type;
        
        // Only load on product edit pages and our settings page
        // phpcs:ignore WordPress.Security.NonceVerification.Recommended
        if (($pagenow == 'post.php' && $post_type == 'product') || 
            ($pagenow == 'post-new.php' && $post_type == 'product') || 
            (isset($_GET['page']) && strpos(sanitize_text_field(wp_unslash($_GET['page'])), 'apr-') === 0)) {
                    
            wp_enqueue_script('jquery-ui-datepicker');
            wp_enqueue_script('jquery-ui-timepicker', NCWCAPR_PLUGIN_URL . 'js/jquery-ui-timepicker-addon.min.js', array('jquery-ui-datepicker'), '1.6.3', true);
            wp_enqueue_style('jquery-ui-css', NCWCAPR_PLUGIN_URL . 'css/jquery-ui.css', array(), '1.12.1');
            wp_enqueue_style('jquery-ui-timepicker-css', NCWCAPR_PLUGIN_URL . 'css/jquery-ui-timepicker-addon.min.css', array('jquery-ui-css'), '1.6.3');
            
            wp_add_inline_script('jquery-ui-timepicker', 'jQuery(document).ready(function($) {
                $("#_restock_date").datetimepicker({ 
                    dateFormat: "yy-mm-dd",
                    timeFormat: "HH:mm:ss"
                });

                // Show/hide restock quantity based on "Track stock quantity for this product" checkbox
                function toggleRestockFields() {
                    if ($("input[name=\'_manage_stock\']").is(":checked")) {
                        $("#_restock_quantity_field").show();
                    } else {
                        $("#_restock_quantity_field").hide();
                    }
                }

                toggleRestockFields();
                $("input[name=\'_manage_stock\']").change(toggleRestockFields);
            });');

            // Add custom CSS
            wp_add_inline_style('jquery-ui-css', '
                #_restock_quantity_field { 
                    display: none; 
                }
                .restock-settings-divider {
                    border-top: 1px solid #eee;
                    margin: 15px 0;
                    padding-top: 15px;
                }
            ');
        }
    }

    /**
     * Add restock date field to product inventory tab
     */
    public function add_restock_date_field() {
        wp_nonce_field('save_restock_date', 'ncwcapr_restock_nonce');

        echo '<div class="options_group">';
        echo '<h4 style="padding-left: 12px;">' . esc_html__('Auto Restock Settings', 'auto-product-restock-for-woo') . '</h4>';
        
        // Restock date and time field
        woocommerce_wp_text_input(array(
            'id' => '_restock_date',
            'label' => esc_html__('Restock Date & Time', 'auto-product-restock-for-woo'),
            'desc_tip' => 'true',
            'description' => esc_html__('Enter the expected restock date and time (YYYY-MM-DD HH:MM:SS).', 'auto-product-restock-for-woo'),
            'class' => 'wc-autostock-date'
        ));
        
        // Restock quantity field
        woocommerce_wp_text_input(array(
            'id' => '_restock_quantity',
            'label' => esc_html__('Restock Quantity', 'auto-product-restock-for-woo'),
            'desc_tip' => 'true',
            'description' => esc_html__('Enter the expected restock quantity.', 'auto-product-restock-for-woo'),
            'type' => 'number',
            'custom_attributes' => array(
                'min' => '0'
            ),
            'wrapper_class' => 'hide_if_no_stock_management'
        ));
        
        echo '</div>'; // Close options_group
    }

    /**
     * Save restock date and quantity fields
     */
    public function save_restock_date_field($post_id) {
        if (!isset($_POST['ncwcapr_restock_nonce']) || !wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['ncwcapr_restock_nonce'])), 'save_restock_date')) {
            return;
        }
        
        // Save restock date and quantity
        if (isset($_POST['_restock_date'])) {
            $restock_date = sanitize_text_field(wp_unslash($_POST['_restock_date']));
            update_post_meta($post_id, '_restock_date', $restock_date);
        }
        
        if (isset($_POST['_restock_quantity'])) {
            $restock_quantity = absint(sanitize_text_field(wp_unslash($_POST['_restock_quantity'])));
            update_post_meta($post_id, '_restock_quantity', $restock_quantity);
        }
    }

    /**
     * Schedule the restock check event
     */
    public function schedule_restock_event() {
        if (!wp_next_scheduled('ncwcapr_restock_check_dates')) {
            wp_schedule_event(time(), 'ncwcapr_restock_interval', 'ncwcapr_restock_check_dates');
        }
    }

    /**
     * Update stock status for products that should be restocked
     */
    public function update_stock_status() {
        $log_enabled = get_option('ncwcapr_restock_enable_logging', 'no');
        $this->log('APR Auto Restock: Starting stock update process', $log_enabled);
        
        // Get current date and time in site's timezone
        $timezone = new DateTimeZone(wp_timezone_string());
        $current_date = new DateTime('now', $timezone);
        $today_date = $current_date->format('Y-m-d');
        $current_time = $current_date->format('H:i:s');
        
        $this->log(
            sprintf('APR Auto Restock: Current date/time: %s %s', $today_date, $current_time),
            $log_enabled
        );
        
        // Process one-time restocks
        $this->process_one_time_restocks($today_date, $current_time, $log_enabled);
        
        // Let other classes handle their own restocks
        do_action('ncwcapr_process_additional_restocks', $current_date, $log_enabled);
    }
    
    /**
     * Process one-time restocks
     */
    private function process_one_time_restocks($today_date, $current_time, $log_enabled) {
        // Query products with one-time restock dates set for today or earlier
        $args = array(
            'post_type' => 'product',
            'post_status' => 'publish',
            'posts_per_page' => -1,
            'fields' => 'ids',
            'meta_query' => array(
                array(
                    'key' => '_restock_date',
                    'value' => '',
                    'compare' => '!='
                )
            ),
        );
        
        $product_ids = get_posts($args);
        $this->log('APR Auto Restock: Found ' . count($product_ids) . ' products with restock dates', $log_enabled);
        
        foreach ($product_ids as $product_id) {
            $restock_datetime = get_post_meta($product_id, '_restock_date', true);
            
            // Skip if empty
            if (empty($restock_datetime)) {
                continue;
            }
            
            // Parse the restock date and time
            $restock_parts = explode(' ', $restock_datetime);
            $restock_date = isset($restock_parts[0]) ? $restock_parts[0] : '';
            $restock_time = isset($restock_parts[1]) ? $restock_parts[1] : '00:00:00';
            
            // Check if we should restock now
            if ($restock_date < $today_date || ($restock_date == $today_date && $restock_time <= $current_time)) {
                $this->restock_product($product_id, $log_enabled);
                
                // Clear the restock date for one-time restocks
                $recurrence_enabled = get_post_meta($product_id, '_restock_recurrence_enabled', true);
                if ($recurrence_enabled !== 'yes') {
                    delete_post_meta($product_id, '_restock_date');
                    $this->log("APR Auto Restock: Cleared one-time restock date for product ID $product_id", $log_enabled);
                }
            }
        }
    }
    
    /**
     * Restock a product
     */
    public function restock_product($product_id, $log_enabled = 'no') {
        $product = wc_get_product($product_id);
        
        if (!$product) {
            $this->log("APR Auto Restock: Invalid product ID $product_id", $log_enabled);
            return;
        }
        
        $restock_quantity = absint(get_post_meta($product_id, '_restock_quantity', true));
        $this->log("APR Auto Restock: Processing restock for product ID $product_id with quantity $restock_quantity", $log_enabled);
        
        if ($product->get_manage_stock()) {
            if ($restock_quantity > 0) {
                // Update the stock quantity
                $product->set_stock_quantity($restock_quantity);
                $product->set_stock_status('instock');
                $product->save();
                
                // Log the event
                $this->log("APR Auto Restock: Updated stock for product ID $product_id to quantity $restock_quantity", $log_enabled);
            }
        } else {
            // Just update the stock status
            update_post_meta($product_id, '_stock_status', 'instock');
            
            // Log the event
            $this->log("APR Auto Restock: Updated stock status to 'instock' for product ID $product_id", $log_enabled);
        }
        
        do_action('ncwcapr_after_product_restock', $product_id, $restock_quantity);
    }

    /**
     * Display restock date on the product page
     */
    public function display_restock_date() {
        global $product;

        if (!$product || !$product instanceof WC_Product) {
            return;
        }

        if (!$product->is_in_stock()) {
            $restock_date = get_post_meta($product->get_id(), '_restock_date', true);
            if ($restock_date) {
                // Parse the datetime
                $formatted_date = date_i18n(get_option('date_format') . ' ' . get_option('time_format'), strtotime($restock_date));
                
                // Translators: %s is the expected restock date and time
                echo '<p class="restock-date">' . sprintf(esc_html__('Expected back in stock: %s', 'auto-product-restock-for-woo'), esc_html($formatted_date)) . '</p>';
            }
        }
    }

    /**
     * Add custom cron intervals
     */
    public function add_cron_intervals($schedules) {
        $interval = absint(get_option('ncwcapr_restock_cron_schedule', 4)) * 3600;
        $schedules['ncwcapr_restock_interval'] = array(
            'interval' => $interval,
            'display' => esc_html__('Auto Restock Interval', 'auto-product-restock-for-woo')
        );
        return $schedules;
    }
    
    /**
    * Log message to the error log if enabled
    */
    public function log($message, $enabled) {
        if ($enabled === 'yes') {
            // phpcs:ignore WordPress.PHP.DevelopmentFunctions.error_log_error_log
            error_log($message);
        }
    }
}