<?php
if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

/**
 * Dashboard Widget class
 * Adds a widget to the WordPress dashboard showing upcoming restocks
 */
class NCWCAPR_Dashboard_Widget {

    /**
     * Constructor
     */
    public function __construct() {
        // Add the dashboard widget
        add_action('wp_dashboard_setup', array($this, 'add_dashboard_widget'));
    }
    
    /**
     * Add dashboard widget
     */
    public function add_dashboard_widget() {
        // Only add for users who can manage WooCommerce
        if (current_user_can('manage_woocommerce')) {
            wp_add_dashboard_widget(
                'apr_upcoming_restocks_widget',
                esc_html__('Upcoming Product Restocks', 'auto-product-restock-for-woo'),
                array($this, 'dashboard_widget_content')
            );
        }
    }
    
    /**
     * Dashboard widget content
     */
    public function dashboard_widget_content() {
        // Get current date
        $today = gmdate('Y-m-d');
        $thirty_days_later = gmdate('Y-m-d', strtotime('+30 days'));
        
        // Query one-time restocks in the next 30 days
        $args = array(
            'post_type' => 'product',
            'post_status' => 'publish',
            'posts_per_page' => 10,
            'meta_query' => array(
                array(
                    'key' => '_restock_date',
                    'value' => array($today, $thirty_days_later),
                    'compare' => 'BETWEEN',
                    'type' => 'DATE'
                ),
            ),
            'orderby' => 'meta_value',
            'meta_key' => '_restock_date',
            'order' => 'ASC'
        );
        
        $upcoming_restocks = new WP_Query($args);
        
        if ($upcoming_restocks->have_posts()) {
            echo '<table class="widefat fixed" cellspacing="0">';
            echo '<thead><tr>';
            echo '<th>' . esc_html__('Product', 'auto-product-restock-for-woo') . '</th>';
            echo '<th>' . esc_html__('Restock Date', 'auto-product-restock-for-woo') . '</th>';
            echo '<th>' . esc_html__('Quantity', 'auto-product-restock-for-woo') . '</th>';
            echo '</tr></thead>';
            echo '<tbody>';
            
            while ($upcoming_restocks->have_posts()) {
                $upcoming_restocks->the_post();
                $product_id = get_the_ID();
                $product = wc_get_product($product_id);
                
                if (!$product) {
                    continue;
                }
                
                $restock_date = get_post_meta($product_id, '_restock_date', true);
                $restock_quantity = get_post_meta($product_id, '_restock_quantity', true);
                $edit_link = get_edit_post_link($product_id);
                
                // Format the datetime
                $formatted_date = date_i18n(get_option('date_format') . ' ' . get_option('time_format'), strtotime($restock_date));
                
                echo '<tr>';
                echo '<td><a href="' . esc_url($edit_link) . '">' . esc_html($product->get_name()) . '</a></td>';
                echo '<td>' . esc_html($formatted_date) . '</td>';
                echo '<td>' . esc_html($restock_quantity) . '</td>';
                echo '</tr>';
            }
            
            echo '</tbody></table>';
            
            // Add view all link
            echo '<p class="textright"><a href="' . esc_url(admin_url('edit.php?post_type=product&page=apr-upcoming-restocks')) . '">' . 
                 esc_html__('View all upcoming restocks', 'auto-product-restock-for-woo') . ' →</a></p>';
        } else {
            echo '<p>' . esc_html__('No upcoming product restocks in the next 30 days.', 'auto-product-restock-for-woo') . '</p>';
        }
        
        wp_reset_postdata();
        
        // Add section for recurring restocks
        echo '<h3>' . esc_html__('Recurring Restocks', 'auto-product-restock-for-woo') . '</h3>';
        
        $recurring_args = array(
            'post_type' => 'product',
            'post_status' => 'publish',
            'posts_per_page' => 5,
            'meta_query' => array(
                array(
                    'key' => '_restock_recurrence_enabled',
                    'value' => 'yes',
                    'compare' => '='
                ),
            ),
            'orderby' => 'title',
            'order' => 'ASC'
        );
        
        $recurring_restocks = new WP_Query($recurring_args);
        
        if ($recurring_restocks->have_posts()) {
            echo '<table class="widefat fixed" cellspacing="0">';
            echo '<thead><tr>';
            echo '<th>' . esc_html__('Product', 'auto-product-restock-for-woo') . '</th>';
            echo '<th>' . esc_html__('Recurrence', 'auto-product-restock-for-woo') . '</th>';
            echo '<th>' . esc_html__('Time', 'auto-product-restock-for-woo') . '</th>';
            echo '<th>' . esc_html__('Quantity', 'auto-product-restock-for-woo') . '</th>';
            echo '</tr></thead>';
            echo '<tbody>';
            
            while ($recurring_restocks->have_posts()) {
                $recurring_restocks->the_post();
                $product_id = get_the_ID();
                $product = wc_get_product($product_id);
                
                if (!$product) {
                    continue;
                }
                
                $recurrence_type = get_post_meta($product_id, '_restock_recurrence_type', true);
                $recurrence_time = get_post_meta($product_id, '_restock_recurrence_time', true);
                $restock_quantity = get_post_meta($product_id, '_restock_quantity', true);
                $edit_link = get_edit_post_link($product_id);
                
                // Format the recurrence info
                $recurrence_info = esc_html__('Unknown', 'auto-product-restock-for-woo');
                
                if ($recurrence_type === 'weekly') {
                    $days = array();
                    $day_names = array(
                        'monday' => esc_html__('Monday', 'auto-product-restock-for-woo'),
                        'tuesday' => esc_html__('Tuesday', 'auto-product-restock-for-woo'),
                        'wednesday' => esc_html__('Wednesday', 'auto-product-restock-for-woo'),
                        'thursday' => esc_html__('Thursday', 'auto-product-restock-for-woo'),
                        'friday' => esc_html__('Friday', 'auto-product-restock-for-woo'),
                        'saturday' => esc_html__('Saturday', 'auto-product-restock-for-woo'),
                        'sunday' => esc_html__('Sunday', 'auto-product-restock-for-woo')
                    );
                    
                    foreach ($day_names as $day_key => $day_value) {
                        if (get_post_meta($product_id, '_restock_day_' . $day_key, true) === 'yes') {
                            $days[] = $day_value;
                        }
                    }
                    
                    $recurrence_info = sprintf(
                        /* translators: %s: Comma-separated list of weekdays */
                        esc_html__('Weekly: %s', 'auto-product-restock-for-woo'),
                        implode(', ', $days)
                    );
                } elseif ($recurrence_type === 'monthly') {
                    $day_of_month = get_post_meta($product_id, '_restock_day_of_month', true);
                    $recurrence_info = sprintf(
                        /* translators: %d: Day of month (1-31) */
                        esc_html__('Monthly: Day %d', 'auto-product-restock-for-woo'),
                        intval($day_of_month)
                    );
                }
                
                // Format time
                $formatted_time = !empty($recurrence_time) ? date_i18n(get_option('time_format'), strtotime($recurrence_time)) : esc_html__('Midnight', 'auto-product-restock-for-woo');
                
                echo '<tr>';
                echo '<td><a href="' . esc_url($edit_link) . '">' . esc_html($product->get_name()) . '</a></td>';
                echo '<td>' . esc_html($recurrence_info) . '</td>';
                echo '<td>' . esc_html($formatted_time) . '</td>';
                echo '<td>' . esc_html($restock_quantity) . '</td>';
                echo '</tr>';
            }
            
            echo '</tbody></table>';
            
            // Add view all link
            echo '<p class="textright"><a href="' . esc_url(admin_url('edit.php?post_type=product&page=apr-recurring-restocks')) . '">' . 
                 esc_html__('View all recurring restocks', 'auto-product-restock-for-woo') . ' →</a></p>';
        } else {
            echo '<p>' . esc_html__('No products with recurring restocks configured.', 'auto-product-restock-for-woo') . '</p>';
        }
        
        wp_reset_postdata();
    }
}