<?php
if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

/**
 * Recurring Restock functionality class
 * Handles recurring restock patterns (weekly, monthly)
 */
class NCWCAPR_Recurring_Restock {

    /**
     * Constructor
     */
    public function __construct() {
        // Hook to add recurring fields to product
        add_action('woocommerce_product_options_inventory_product_data', array($this, 'add_recurring_fields'), 20);
        
        // Hook to save recurring fields
        add_action('woocommerce_process_product_meta', array($this, 'save_recurring_fields'), 20);
        
        // Hook to add our own processing for recurring restocks
        add_action('ncwcapr_process_additional_restocks', array($this, 'process_recurring_restocks'), 10, 2);
        
        // Enqueue scripts and styles
        add_action('admin_enqueue_scripts', array($this, 'enqueue_scripts'));
    }
    
    /**
     * Enqueue scripts for recurring restock fields
     */
    public function enqueue_scripts() {
        global $pagenow, $post_type;
        
        // Only load on product edit pages
        if (($pagenow == 'post.php' && $post_type == 'product') || 
            ($pagenow == 'post-new.php' && $post_type == 'product')) {
            
            wp_add_inline_script('jquery-ui-timepicker', 'jQuery(document).ready(function($) {
                // Initialize the time picker for recurring restock time
                $("#_restock_recurrence_time").timepicker({
                    timeFormat: "HH:mm:ss"
                });
                
                // Show/hide recurrence fields based on recurrence enabled checkbox
                function toggleRecurrenceFields() {
                    if ($("#_restock_recurrence_enabled").is(":checked")) {
                        $(".restock-recurrence-field").show();
                        updateRecurrenceFields();
                    } else {
                        $(".restock-recurrence-field").hide();
                    }
                }
                
                function updateRecurrenceFields() {
                    var type = $("#_restock_recurrence_type").val();
                    $(".recurrence-weekly-fields, .recurrence-monthly-fields").hide();
                    
                    if (type === "weekly") {
                        $(".recurrence-weekly-fields").show();
                    } else if (type === "monthly") {
                        $(".recurrence-monthly-fields").show();
                    }
                }

                toggleRecurrenceFields();
                $("#_restock_recurrence_enabled").change(toggleRecurrenceFields);
                $("#_restock_recurrence_type").change(updateRecurrenceFields);
            });');

            // Add custom CSS
            wp_add_inline_style('jquery-ui-css', '
                .restock-recurrence-field { 
                    display: none; 
                }
                .recurrence-weekly-fields,
                .recurrence-monthly-fields {
                    display: none;
                    margin-left: 20px;
                    padding-top: 10px;
                }
                .recurrence-day-label {
                    display: inline-block;
                    margin-right: 10px;
                }
            ');
        }
    }
    
    /**
     * Add recurring restock fields to product inventory tab
     */
    public function add_recurring_fields() {

        wp_nonce_field('save_recurring_fields', 'ncwcapr_recurring_nonce');

        echo '<div class="options_group">';
        echo '<div class="restock-settings-divider"></div>';
        echo '<h4 style="padding-left: 12px;">' . esc_html__('Recurring Restock Settings', 'auto-product-restock-for-woo') . '</h4>';
        
        // Recurrence enabled checkbox
        woocommerce_wp_checkbox(array(
            'id' => '_restock_recurrence_enabled',
            'label' => esc_html__('Enable Recurring Restock', 'auto-product-restock-for-woo'),
            'description' => esc_html__('Enable recurring restocking for this product.', 'auto-product-restock-for-woo')
        ));
        
        // Recurrence time 
        woocommerce_wp_text_input(array(
            'id' => '_restock_recurrence_time',
            'label' => esc_html__('Restock Time', 'auto-product-restock-for-woo'),
            'desc_tip' => true,
            'description' => esc_html__('Set the time when the restock should occur (HH:MM:SS).', 'auto-product-restock-for-woo'),
            'placeholder' => '00:00:00',
            'wrapper_class' => 'restock-recurrence-field'
        ));
        
        // Recurrence type selection
        woocommerce_wp_select(array(
            'id' => '_restock_recurrence_type',
            'label' => esc_html__('Recurrence Pattern', 'auto-product-restock-for-woo'),
            'options' => array(
                '' => esc_html__('Select a pattern', 'auto-product-restock-for-woo'),
                'weekly' => esc_html__('Weekly', 'auto-product-restock-for-woo'),
                'monthly' => esc_html__('Monthly', 'auto-product-restock-for-woo')
            ),
            'desc_tip' => true,
            'description' => esc_html__('Select the recurrence pattern for restocking.', 'auto-product-restock-for-woo'),
            'wrapper_class' => 'restock-recurrence-field'
        ));
        
        // Weekly recurrence options
        echo '<div class="recurrence-weekly-fields restock-recurrence-field">';
        
        $days_of_week = array(
            'monday' => esc_html__('Monday', 'auto-product-restock-for-woo'),
            'tuesday' => esc_html__('Tuesday', 'auto-product-restock-for-woo'),
            'wednesday' => esc_html__('Wednesday', 'auto-product-restock-for-woo'),
            'thursday' => esc_html__('Thursday', 'auto-product-restock-for-woo'),
            'friday' => esc_html__('Friday', 'auto-product-restock-for-woo'),
            'saturday' => esc_html__('Saturday', 'auto-product-restock-for-woo'),
            'sunday' => esc_html__('Sunday', 'auto-product-restock-for-woo')
        );
        
        echo '<p>' . esc_html__('Select days for weekly restock:', 'auto-product-restock-for-woo') . '</p>';
        
        foreach ($days_of_week as $day_key => $day_label) {
            echo '<div class="recurrence-day-label">';
            woocommerce_wp_checkbox(array(
                'id' => '_restock_day_' . $day_key,
                'label' => $day_label,
                'cbvalue' => 'yes',
                'wrapper_class' => 'form-field checkbox_field'
            ));
            echo '</div>';
        }
        
        echo '</div>';
        
        // Monthly recurrence options
        echo '<div class="recurrence-monthly-fields restock-recurrence-field">';
        
        $monthly_option = array();
        for ($i = 1; $i <= 31; $i++) {
            /* translators: %d: Day of month (1-31) */
            $monthly_option[$i] = sprintf(esc_html__('Day %d of month', 'auto-product-restock-for-woo'), $i);
        }
        
        woocommerce_wp_select(array(
            'id' => '_restock_day_of_month',
            'label' => esc_html__('Day of Month', 'auto-product-restock-for-woo'),
            'options' => $monthly_option,
            'desc_tip' => true,
            'description' => esc_html__('Select which day of the month to restock.', 'auto-product-restock-for-woo')
        ));
        
        echo '</div>';
        
        echo '</div>'; // Close options_group
    }
    
    /**
     * Save recurring restock fields
     */
    public function save_recurring_fields($post_id) {
    // Verify nonce
        if (!isset($_POST['ncwcapr_recurring_nonce']) || !wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['ncwcapr_recurring_nonce'])), 'save_recurring_fields')) {
            return;
        }
        
        // Save recurrence settings
        $recurrence_enabled = isset($_POST['_restock_recurrence_enabled']) ? 'yes' : 'no';
        update_post_meta($post_id, '_restock_recurrence_enabled', $recurrence_enabled);
        
        // Save recurrence time
        if (isset($_POST['_restock_recurrence_time'])) {
            $time = sanitize_text_field(wp_unslash($_POST['_restock_recurrence_time']));
            // Default to midnight if empty
            if (empty($time)) {
                $time = '00:00:00';
            }
            update_post_meta($post_id, '_restock_recurrence_time', $time);
        }
        
        if (isset($_POST['_restock_recurrence_type'])) {
            $recurrence_type = sanitize_text_field(wp_unslash($_POST['_restock_recurrence_type']));
            update_post_meta($post_id, '_restock_recurrence_type', $recurrence_type);
        }
        
        // Save weekly recurrence days
        $days_of_week = array('monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sunday');
        foreach ($days_of_week as $day) {
            $day_key = '_restock_day_' . $day;
            $day_value = isset($_POST[$day_key]) ? 'yes' : 'no';
            update_post_meta($post_id, $day_key, $day_value);
        }
        
        // Save monthly recurrence day
        if (isset($_POST['_restock_day_of_month'])) {
            $day_of_month = absint(sanitize_text_field(wp_unslash($_POST['_restock_day_of_month'])));
            update_post_meta($post_id, '_restock_day_of_month', $day_of_month);
        }
    }
    
    /**
     * Process recurring restocks
     */
    public function process_recurring_restocks($current_date, $log_enabled) {
        // Current day of week (1 = Monday, 7 = Sunday)
        $day_of_week = $current_date->format('N');
        $day_names = array(
            1 => 'monday',
            2 => 'tuesday',
            3 => 'wednesday',
            4 => 'thursday',
            5 => 'friday',
            6 => 'saturday',
            7 => 'sunday'
        );
        $current_day_name = $day_names[$day_of_week];
        
        // Current day of month (1-31)
        $day_of_month = $current_date->format('j');
        $current_time = $current_date->format('H:i:s');
        
        $this->log(
            sprintf(
                'APR Recurring Restock: Processing for day: %s, Day of month: %s, Current time: %s',
                $current_day_name,
                $day_of_month,
                $current_time
            ),
            $log_enabled
        );
        
        // Query products with recurring restocks enabled
        $args = array(
            'post_type' => 'product',
            'post_status' => 'publish',
            'posts_per_page' => -1,
            'fields' => 'ids',
            'meta_query' => array(
                array(
                    'key' => '_restock_recurrence_enabled',
                    'value' => 'yes',
                    'compare' => '='
                )
            ),
        );
        
        $product_ids = get_posts($args);
        $this->log('APR Recurring Restock: Found ' . count($product_ids) . ' products with recurring restocks enabled', $log_enabled);
        
        foreach ($product_ids as $product_id) {
            $recurrence_type = get_post_meta($product_id, '_restock_recurrence_type', true);
            $recurrence_time = get_post_meta($product_id, '_restock_recurrence_time', true);
            
            // Default to midnight if no time is set
            if (empty($recurrence_time)) {
                $recurrence_time = '00:00:00';
            }
            
            // Check if we've reached or passed the restock time
            if ($current_time < $recurrence_time) {
                $this->log("APR Recurring Restock: Not yet time to restock product ID $product_id (current: $current_time, restock: $recurrence_time)", $log_enabled);
                continue;
            }
            
            if ($recurrence_type === 'weekly') {
                // Check if today is a selected day for restock
                $today_selected = get_post_meta($product_id, '_restock_day_' . $current_day_name, true);
                
                if ($today_selected === 'yes') {
                    $last_restock = get_post_meta($product_id, '_last_recurring_restock', true);
                    $today = $current_date->format('Y-m-d');
                    
                    // Only restock if not already done today
                    if ($last_restock !== $today) {
                        $this->process_restock($product_id, $log_enabled);
                        update_post_meta($product_id, '_last_recurring_restock', $today);
                    } else {
                        $this->log("APR Recurring Restock: Product ID $product_id already restocked today", $log_enabled);
                    }
                }
            } elseif ($recurrence_type === 'monthly') {
                // Check if today is the selected day of month
                $selected_day = (int) get_post_meta($product_id, '_restock_day_of_month', true);
                
                if ($day_of_month == $selected_day) {
                    $last_restock = get_post_meta($product_id, '_last_recurring_restock', true);
                    $today = $current_date->format('Y-m-d');
                    
                    // Only restock if not already done today
                    if ($last_restock !== $today) {
                        $this->process_restock($product_id, $log_enabled);
                        update_post_meta($product_id, '_last_recurring_restock', $today);
                    } else {
                        $this->log("APR Recurring Restock: Product ID $product_id already restocked today", $log_enabled);
                    }
                }
            }
        }
    }
    
    /**
     * Process a single product restock
     */
    private function process_restock($product_id, $log_enabled) {
        global $ncwcapr_auto_restock;
        
        // If the main plugin class exists, use its restock method
        if (isset($GLOBALS['ncwcapr_auto_restock']) && $GLOBALS['ncwcapr_auto_restock'] instanceof NCWCAPR_Auto_Restock) {
            $GLOBALS['ncwcapr_auto_restock']->restock_product($product_id, $log_enabled);
        } else {
            // Fallback implementation if the main class is not available
            $product = wc_get_product($product_id);
            
            if (!$product) {
                $this->log("APR Recurring Restock: Invalid product ID $product_id", $log_enabled);
                return;
            }
            
            $restock_quantity = absint(get_post_meta($product_id, '_restock_quantity', true));
            $this->log("APR Recurring Restock: Processing restock for product ID $product_id with quantity $restock_quantity", $log_enabled);
            
            if ($product->get_manage_stock()) {
                if ($restock_quantity > 0) {
                    // Update the stock quantity
                    $product->set_stock_quantity($restock_quantity);
                    $product->set_stock_status('instock');
                    $product->save();
                    
                    // Log the event
                    $this->log("APR Recurring Restock: Updated stock for product ID $product_id to quantity $restock_quantity", $log_enabled);
                }
            } else {
                // Just update the stock status
                update_post_meta($product_id, '_stock_status', 'instock');
                
                // Log the event
                $this->log("APR Recurring Restock: Updated stock status to 'instock' for product ID $product_id", $log_enabled);
            }
        }
    }
    
    /**
     * Log message to the error log if enabled
     */
    private function log($message, $enabled) {
        if ($enabled === 'yes') {
            error_log($message);
        }
    }
}